// Copyright (C) 2010 - 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

function Y = opustc_couplex ( varargin )
    //   Returns the output for the OPUS - COUPLEX test case.
    //
    // Calling Sequence
    //   Y = opustc_couplex ( X )
    //   Y = opustc_couplex ( X , D )
    //
    // Parameters
    // X: a np-by-15 matrix, where np is the number of experiments to perform
    // D: a parameter list with the determinist parameters
    // D.directory: the subdirectory of the temporary directory where the simulation must be performed (default = "couplex")
    // D.verbose: set to true to make the simulation verbose (default %f)
    // D.castembin: the name of the CASTEM executable (default "castem09")
    // Y: a np-by-168 matrix
    //
    // Description
    //   Launch Cast3m and perform the COUPLEX simulation.
    //
    //   Prerequirements: 
    //   The Cast3m software must be installed on the machine.
    //
    //   http://www-cast3m.cea.fr/cast3m/index.jsp
    //
    //   In order to check that cast3m is correctly installed on your
    //   system, open a terminal and type, on Windows:
    //
    //   castem09.bat
	//
	// on Linux:
    //
    //   castem09
    //   
    //   The data associated with this benchmark is provided in the 
    //   "data/couplex" subdirectory of this module.
    //   This directory contains the following files:
    //         <itemizedlist>
    //           <listitem>
    //             <para>
    //               couplex.dgibi.ref (input file of CASTEM): contains the template couplex input file for 
    //               CASTEM. The content of this file is modified, so that the 
    //               actual data correspond to the input X.
    //             </para>
    //           </listitem>
    //           <listitem>
    //             <para>
    //               filePlanExperience.txt (input file of CASTEM): the design of experiments data file.
    //               Each row of this file correspond to an experiment to perform.
    //               Each column of this file correspond to an input parameter.
    //             </para>
    //           </listitem>
    //           <listitem>
    //             <para>
    //               iode.dat (input file of CASTEM): the temporal evolution of the iode, the source 
    //               term of the partial differential equation.
    //             </para>
    //           </listitem>
    //           <listitem>
    //             <para>
    //               fileReponse.txt (output file of CASTEM): the output of the simulation.
    //               Each row of this file correspond to an experiment which has been performed.
    //               Each column of this file correspond to an output parameter.
    //             </para>
    //           </listitem>
    //           <listitem>
    //             <para>
    //               resu_conc.dat (output file of CASTEM): the concentration of different 
    //               species depending on the time and the point of measure (P1 to P8).
    //             </para>
    //           </listitem>
    //           <listitem>
    //             <para>
    //               resu_flux.dat (output file of CASTEM): the flux of iode through 
    //               different surfaces of the mesh.
    //             </para>
    //           </listitem>
    //         </itemizedlist>
	//
    // See the data/couplex/opuscouplex.pdf document for a more complete 
    // manual of this benchmark.
	//
	// CASTEM prints messages in the console. 
	// In some cases, CASTEM cannot read the data files. 
	// In this case, CASTEM reads from the console interactively,
	// which obviously fails.
	// To detect these situations, open the console where Scilab runs in
	// with the consolebox function or the "F12" Scilab shortcut.
    //
    // Examples
	// // Check that the "CASTEM" environment variable is well defined
	// getenv("CASTEM")
	//
	// // CASTEM prins messages in the console.
	// consolebox("on")
	// // or F12.
	//
	// // Perform one experiment
    // X = [25.0 3.0e-6 6.0 3.0e-5 1e-6 5.0e-4 50. 50. 50. 50. 21. 51. 3. 110. 160. ];
    // Y = opustc_couplex ( X )
    //
    // // Enable verbose logging:
    // D = init_param();
    // D = add_param(D,"verbose",%t);
    // Y = opustc_couplex ( X , D );
    //
    // // Customize directory
    // D = init_param();
    // D = add_param(D,"directory","couplex_1");
    // D = add_param(D,"verbose",%t);
    // Y = opustc_couplex ( X , D );
    //
    // // See the data associated with the couplex benchmark.
    // // This data is in the "data/couplex" subdirectory of this module.
    // path = opustc_getpath ( );
    // ls(fullfile(path,"data","couplex"))
    //
    // Authors
    //   Copyright (C) 2010 - 2011 - INRIA - Michael Baudin
    //
    // Bibliography
    //   "OPUS, Cas test CEA, Propagation des incertitudes dans le probleme COUPLEX-1", JM. Martinez, Ph. Montarnal, M. Lamoureux, O. Le Maitre, 6 avril 2010
    //   "Analyse d'incertitudes : application de la theorie des polynomes du chaos a l'analyse de surete du stockage de dechets nucleaires", Ecole Centrale Paris, Paul Balanca, Anne-Celine Boulanger, Marc Dupin, Cyril Labbe, Etude encadree par Florian De Vuyst
    //

    [lhs, rhs] = argn()
    apifun_checkrhs ( "opustc_couplex" , rhs , 1:2 )
    apifun_checklhs ( "opustc_couplex" , lhs , 1:1 )
    //
    // Get arguments
    X = varargin(1)
    if ( rhs < 2 ) then
        D = init_param();
    else
        D = varargin(2)
    end
    //
    // Check types
    apifun_checktype ( "opustc_couplex" , X , "X" , 1 , "constant" )
    apifun_checktype ( "opustc_couplex" , D , "D" , 2 , "plist" )
    //
    // Check size
    np = size(X,"r")
    apifun_checkdims ( "opustc_couplex" , X , "X" , 1 , [np 15] )
    //
    // Get parameters
    [verbose,err] = get_param(D,"verbose",%f)
    [simulationdir,err] = get_param(D,"directory","couplex")
    //
    // Check types
    apifun_checktype ( "opustc_couplex" , verbose , "D.verbose" , 2 , "boolean" )
    apifun_checktype ( "opustc_couplex" , simulationdir , "D.simulationdir" , 2 , "string" )
    //
    // Check size
    apifun_checkscalar ( "opustc_couplex" , verbose , "D.verbose" , 2 )
    apifun_checkscalar ( "opustc_couplex" , simulationdir , "D.simulationdir" , 2 )
    //
    // 1. Setup a directory, with data files
    [cplxdir,plexfile,outputfile, castembin , dgibifile ] = opustc_couplexstartup ( X , D )
    if ( verbose ) then
        w=getdate()
        ststart = timer();
        utstart = tic();
        mprintf("Start at %d/%d/%d at %d:%d:%d\n",w(6),w(2),w(1),w(7),w(8),w(9));
    end
    //
    // 2. Make the simulation
    cwd = pwd()
    chdir ( cplxdir ) ;
	if ( verbose ) then
		verbosemode = 1;
	else
		verbosemode = 0;
	end
    opustc_couplexrun ( plexfile , outputfile , castembin , dgibifile , verbosemode )
    if ( verbose ) then
        w=getdate()
        mprintf("End at %d/%d/%d at %d:%d:%d\n",w(6),w(2),w(1),w(7),w(8),w(9));
        stend = timer();
        mprintf("System time: %f (s)\n",stend)
        utend = toc();
        mprintf("User time: %f (s)\n",utend)
        mprintf("Reading output file: %s...\n", outputfile)
    end
    //
    // 3. Get the output
    Y = fscanfMat(outputfile)
    chdir ( cwd ) ;
    if ( verbose ) then
        mprintf("Done.\n")
    end
endfunction

