// Copyright (C) 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

function [cplxdir,plexfile,outputfile,castembin,dgibifile] = opustc_couplexstartup ( varargin )
    // Startup the couplex test case, given X.
    //
    // Calling Sequence
    //   [cplxdir,plexfile,outputfile,castembin,dgibifile] = opustc_couplexstartup ( X )
    //   [cplxdir,plexfile,outputfile,castembin,dgibifile] = opustc_couplexstartup ( X , D )
    //
    // Parameters
    // X: a np-by-15 matrix, where np is the number of experiments to perform
    // D: a parameter list with the determinist parameters
    // D.directory: the subdirectory of the temporary directory where the simulation must be performed (default = "couplex")
    // D.verbose: set to true to make the simulation verbose (default %f)
    // D.castembin: the short name of the CASTEM executable (default "castem09")
    // cplxdir : a 1-by-1 matrix of strings, the simulation directory
    // plexfile : a 1-by-1 matrix of strings, the design of experiments file
    // outputfile : a 1-by-1 matrix of strings, the output file
	// castembin : a 1-by-1 matrix of strings, the full path to the CASTEM executable
	// dgibifile : a 1-by-1 matrix of strings, the full path to the dgibi template data file.
    //
    // Description
    //   Prepare the files to startup the COUPLEX test case.
    //
    //   We suppose that the "CASTEM" environment variable contains the path
    //   to the CASTEM executable.
    //
    // Examples
	// // Check that the "CASTEM" environment variable is well defined
	// getenv("CASTEM")
	//
	// // Startup a simulation
    // X = [25.0 3.0e-6 6.0 3.0e-5 1e-6 5.0e-4 50. 50. 50. 50. 21. 51. 3. 110. 160. ];
    // [cplxdir,plexfile,outputfile,castembin,dgibifile] = opustc_couplexstartup ( X )
    //
    // // Customize verbose
    // D = init_param();
    // D = add_param(D,"verbose",%t);
    // X = [25.0 3.0e-6 6.0 3.0e-5 1e-6 5.0e-4 50. 50. 50. 50. 21. 51. 3. 110. 160. ];
    // [cplxdir,plexfile,outputfile,castembin,dgibifile] = opustc_couplexstartup ( X , D )
	//
    // Authors
    //   Michael Baudin, 2011
    //
    // Bibliography
    //   "OPUS, Cas test CEA, Propagation des incertitudes dans le probleme COUPLEX-1", JM. Martinez, Ph. Montarnal, M. Lamoureux, O. Le Maitre, 6 avril 2010
    //   "Analyse d'incertitudes : application de la theorie des polynomes du chaos a l'analyse de surete du stockage de dechets nucleaires", Ecole Centrale Paris, Paul Balanca, Anne-Celine Boulanger, Marc Dupin, Cyril Labbe, Etude encadree par Florian De Vuyst
    //

    [lhs, rhs] = argn()
    apifun_checkrhs ( "opustc_couplexstartup" , rhs , 1:2 )
    apifun_checklhs ( "opustc_couplexstartup" , lhs , 1:5 )
    //
    // Get arguments
    X = varargin(1)
    if ( rhs < 2 ) then
        D = init_param();
    else
        D = varargin(2)
    end
    //
    // Check types
    apifun_checktype ( "opustc_couplexstartup" , X , "X" , 1 , "constant" )
    apifun_checktype ( "opustc_couplexstartup" , D , "D" , 2 , "plist" )
    //
    // Check size
    np = size(X,"r")
    apifun_checkdims ( "opustc_couplexstartup" , X , "X" , 1 , [np 15] )
    //
    // Get parameters
    [verbose,err] = get_param(D,"verbose",%f)
    [simulationdir,err] = get_param(D,"directory","couplex")
    [castembin,err] = get_param(D,"castembin","castem09")
    //
    // Check types
    apifun_checktype ( "opustc_couplex" , verbose , "D.verbose" , 2 , "boolean" )
    apifun_checktype ( "opustc_couplex" , simulationdir , "D.simulationdir" , 2 , "string" )
    //
    // Check size
    apifun_checkscalar ( "opustc_couplex" , verbose , "D.verbose" , 2 )
    apifun_checkscalar ( "opustc_couplex" , simulationdir , "D.simulationdir" , 2 )
    //
    // Name of the parameters
    par_names = [ "PerDog" , "PerCla" , "PerLim" , "PerMar" , "DifCla" , ..
    "DifMar" , "DiLoDo" , "DiLoLi" , "DiTrDo" , "DiTrLi" , ..
    "DeCH21" , "DeCH31" , "DeCH16" , "DeCH25" , "DeCH34" ]' ;
    //
    path = opustc_getpath ( )
    optcdir = fullfile(path,"data")
    //
    // 1. Prepare a computation directory
    cplxdir = fullfile(TMPDIR,simulationdir)
    if ( verbose ) then
        mprintf("Directory: %s\n",cplxdir)
    end
    mkdir ( cplxdir )
    //
    // 2. Get the binary path from the environment
    castemenv = getenv("CASTEM")
    if ( verbose ) then
        mprintf("CASTEM environment variable: %s\n",castemenv)
    end
    // TODO : how to make it independent from the castem release ?
    if ( getos() == "Windows" ) then
        castembin = fullfile(castemenv , "bin" , castembin + ".bat")
    else
        castembin = fullfile(castemenv , "bin" , castembin )
    end
    if ( verbose ) then
        mprintf("Castem binary: %s\n",castembin)
    end
    //
    // 3. Copy the iode.dat data file
    iodefile = fullfile(optcdir,"couplex","iode.dat")
    [status,message] = copyfile ( iodefile , cplxdir )
    if ( status==0 ) then
        error(msprintf(gettext("%s: Failed to copy iode.dat data file: %s"),"opustc_couplexstartup",message))
    end
	//
    // 4. Create the dgibi file name
	dgibifile = fullfile(optcdir,"couplex","couplex.dgibi.ref")
    //
    // 5. Create the filePlanExperience.txt
    plexfile = fullfile(cplxdir,"filePlanExperience.txt")
    if ( verbose ) then
        mprintf("Creating %s...\n", plexfile)
    end
    handle = mopen ( plexfile , "w" ) ;
    mfprintf ( handle , "#Parametres 15\n" )
    mfprintf ( handle , "%s\n" , strcat ( par_names' , " " ) )
    mfprintf ( handle , "#Simulations %d\n",np )
    for ip = 1 : np
        for ix = 1 : 15
            mfprintf ( handle , "%.17e " , X(ip,ix) ) ;
        end
        mfprintf ( handle , "\n" ) ;
    end
    mfprintf ( handle , "\n" ) ;
    mclose ( handle ) ;
    //
    // 6. Create an output file name
    outputfile = fullfile(cplxdir,"fileReponse.txt")
endfunction

